/***************************************************************************
 *                                                                         *
 *                               RasTop                                    *
 *                                                                         *
 *  Version:   1.3 based on RasMol 2.7.1                                   *
 *  ========   June 1st, 2000                                              *
 *                                                                         *
 *  Author:    Philippe Valadon                                            *
 *  =======                                                                *
 *                                                                         *
 *  RasTop is an interactive software to visualize molecular structures    *
 *  based on RasMol Molecular Visualization Tool scripting code. RasTop    *
 *  wraps a friendly graphic interface around RasMol core engine and is    *
 *  intended to promote the use of RasMol as a langage to communicate,     *
 *  transfert, exchange information or teaching purpose among the          *
 *  scientific community.                                                  *
 *                                                                         *
 *  RasTop source is distributed freely. Before using RasTop, contributing *
 *  to the source, or using any of its material, you must read and comply  *
 *  with the NOTICE located in the Help folder of RasTop 1.3 package.      *
 *  The NOTICE applies to RasTop 1.3 as a whole and to the works included  *
 *  within it.                                                             *
 *                                                                         *
 *  Copyrights:                                                            *
 *  ==========                                                             *
 *  RasTop                                                                 *
 *  ------                                                                 *
 *  RasTop version 1.3, August 2000                                        *
 *  Philippe Valadon, P.O. Box 13014, La Jolla, CA 92039, USA              *
 *  valadon@ix.netcom.com                                                  *
 *  Copyright (C) Philippe Valadon 2000                                    *
 *                                                                         *
 *  RasMol                                                                 *
 *  ------                                                                 *
 *  RasMol 2.6 by Roger Sayle                                              *
 *  Biomolecular Structures Group, Glaxo Wellcome Research & Development,  *
 *  Stevenage, Hertfordshire, UK                                           *
 *  Version 2.6, August 1995, Version 2.6.4, December 1998                 *
 *  Copyright (C) Roger Sayle 1992-1999                                    *
 *                                                                         *
 *  Mods by Arne Mueller                                                   *
 *  Version 2.6x1, May 1998                                                *
 *  Copyright (C) Arne Mueller 1998                                        *
 *                                                                         *
 *  Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein                      *
 *  Bernstein + Sons, P.O. Box 177, Bellport, NY, USA                      *
 *  yaya@bernstein-plus-sons.com                                           *
 *  2.7.0 March 1999, 2.7.1 June 1999                                      *
 *  Copyright (C) Herbert J. Bernstein 1998-1999                           *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  If you would like to use major pieces of RasTop in other programs,     *
 *  make modifications to RasTop, or in some other way make what a lawyer  *
 *  would call a "derived work", you are not only permitted to do so, you  *
 *  are encouraged to do so to the condition of respecting the following   *
 *  points:                                                                *
 *   - explain in your documentation how what you did differs              *
 *     from this version of RasTop;                                        *
 *   - make your modified source code available;                           *
 *   - comply with respect to RasTop NOTICE.                               *
 *                                                                         *
 *  Read the full NOTICE for complete information.                         *
 *                                                                         *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// datadisplay.h : header file
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DATADISPLAY_H_INCLUDED_)
#define AFX_DATADISPLAY_H_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

extern "C"
{
#include "rasmol.h"
#include "molecule.h"
#include "abstree.h"
#include "command.h"
#include "outfile.h"
#include "pixutils.h"
#include "script.h"
#include "render.h"
#include "transfor.h"
#include "cmndline.h"
#include "infile.h"
#include "repres.h"
}

/////////////////////////////////////////////////////////////////////////////
// CData 

class CData : public CObject
{
public:
	CData();
	DECLARE_DYNCREATE(CData)

// Attributes
	/*Graphics*/
public:	
LPVOID Data_Database;
int Data_XRange;
int Data_YRange;
int Data_BackR, Data_BackG, Data_BackB;

protected:
double Data_DialValue[9];

	/*molecule*/
CString Data_Info_filename;
CString Data_Info_moleculename;
CString Data_Info_classification;
CString Data_Info_date;
CString Data_Info_technique;
CString Data_Info_identcode;

CString Data_Info_spacegroup;
Real Data_Info_cellalpha;
Real Data_Info_cellbeta;
Real Data_Info_cellgamma;
Real Data_Info_cella;
Real Data_Info_cellb;
Real Data_Info_cellc;

double Data_Info_vecf2o[3];
double Data_Info_veco2f[3];
double Data_Info_matf2o[3][3];
double Data_Info_mato2f[3][3];
double Data_Info_cell[6];

Long Data_Info_bondcount;
int Data_Info_chaincount;
int Data_Info_ssbondcount;
int Data_Info_hbondcount;
int Data_cisbondcount;

int Data_Info_structsource;
int Data_Info_laddercount;
int Data_Info_helixcount;
int Data_Info_turncount;

Molecule	*Data_CurMolecule;
Chain		*Data_CurChain;
Group		*Data_CurGroup;
Atom		*Data_CurAtom;

short Data_UserMask_radius[MAXMASK];
CString  Data_UserMask_mask[MAXMASK];
Byte  Data_UserMask_flags[MAXMASK];
Byte  Data_UserMask_r[MAXMASK];
Byte  Data_UserMask_g[MAXMASK];
Byte  Data_UserMask_b[MAXMASK];

char Data_Residue[MAXRES - MINRES][4];
char Data_ElemDesc[MAXELEM - MINELEM][4];

int Data_MainGroupCount;
int	Data_HetaGroupCount;
Long Data_HetaAtomCount;
Long Data_MainAtomCount;  
int Data_CisBondCutOff;

Long Data_MinX;
Long Data_MinY;
Long Data_MinZ;
Long Data_MaxX;
Long Data_MaxY;
Long Data_MaxZ;

int Data_HMinMaxFlag;
int Data_MMinMaxFlag;
int Data_MinMainTemp;
int Data_MaxMainTemp;
int Data_MinHetaTemp;
int Data_MaxHetaTemp;
int Data_MinMainRes;
int	Data_MaxMainRes;
int Data_MinHetaRes;
int	Data_MaxHetaRes;
int Data_MinAltl;
int Data_MaxAltl;

short Data_MinModel;
short Data_MaxModel;

Long Data_MinHBondDist;
Long Data_MaxHBondDist;
Long Data_MinBondDist;
Long Data_MaxBondDist;
int Data_ElemNo;
int Data_ResNo;
int Data_HasHydrogen;
int Data_MaskCount;
int Data_NMRModel;
int Data_NullBonds;
//int Data_MarkAtoms;

int Data_HBondChainsFlag;

	/*Abstree*/
SymEntry *Data_SymbolTable;

	/*pixutils*/
int Data_SplineCount;
int Data_FontPS;
int Data_FontSize;
int Data_FontStroke;

	/*Script*/
int Data_KinemageFlag;

	/*Command*/
int Data_DataFileFormat;
CString Data_DataFileName;
int	Data_CommandActive;
Long Data_SelectCount;
int	Data_Interactive;

int Data_CalcBondsFlag;

	/*Paused scripts*/
int Data_FileDepth;
int Data_IsPaused;
char Data_NameStack[STACKSIZE][1024];
int Data_LineStack[STACKSIZE];
fpos_t Data_PosStack[STACKSIZE];
char Data_AcceptData[STACKSIZE];

	/*Transfor*/
ShadeDesc Data_Shade[LastShade];
Real Data_RotX[3];
Real Data_RotY[3];
Real Data_RotZ[3];
Long Data_CenX;
Long Data_CenY;
Long Data_CenZ;
Long Data_Cenx;
Long Data_Ceny;
Long Data_Cenz;
Long Data_ShiftS;
Real Data_LastRX;
Real Data_LastRY;
Real Data_LastRZ;

Real Data_MatX[3];
Real Data_MatY[3];
Real Data_MatZ[3];
Real Data_InvX[3];
Real Data_InvY[3];
Real Data_InvZ[3];
Long Data_OrigCX;
Long Data_OrigCY;
Long Data_OrigCZ;

int Data_ScaleCount;
int Data_AltlColours[AltlDepth];

int Data_FakeSpecular ;
int Data_SpecPower;
int Data_ShadePower;
int Data_DotR,Data_DotG,Data_DotB;
int Data_LabR, Data_LabG, Data_LabB;
int Data_BoxR, Data_BoxG, Data_BoxB;
int Data_UseLabelCol;
int Data_UseBackFade;
Real Data_Ambient;
int Data_UseDotColPot;

//Real Data_Scale;
Real Data_MaxZoom;
Real Data_DScale;
Real Data_IScale;
Long Data_SideLen;
Long Data_Offset;
Card Data_WorldRadius;
Card Data_WorldSize;
int Data_XOffset;
int Data_YOffset;
int Data_ZOffset;
int Data_UseScreenClip;
int Data_ZoomRange;

int Data_Hydrogens;
int Data_HetaGroups;
int Data_DrawAtoms;
int Data_MaxAtomRadius;
int Data_DrawBonds;
int Data_MaxBondRadius;
int Data_DrawRibbon;
int Data_DrawStars;
int Data_ZoneBoth;

	/*Render*/
int Data_UseDepthCue;
int Data_UseStereo;
int Data_StereoView;
int Data_UseShadow;
int Data_DisplayMode;
int Data_UseClipping;
int Data_UseSlabPlane;
int Data_UseDepthPlane;
int Data_SlabMode;
int Data_SlabValue;
int Data_SlabInten;
int Data_SliceValue;
int Data_ImageRadius;
int Data_ImageSize;
int Data_SSBondMode;
int Data_HBondMode;
double Data_StereoAngle;
int Data_DrawBoundBox;
int Data_DrawAxes;
int Data_DrawDoubleBonds;
int Data_DrawUnitCell;
Real Data_IVoxRatio;
int Data_VoxelsClean;

	/*Repres*/
DotStruct *Data_DotPtr;
Monitor *Data_MonitList;
Label *Data_LabelList;
int Data_CartoonHeight;
int Data_SolventDots;
int Data_ProbeRadius;
int Data_DotDensity;
int Data_DotSize;
int Data_DrawMonitDistance;
int Data_DrawBetaArrows;

	/*Outfile*/
int Data_UseTransparent;
int Data_UseOutLine;

// Operations
public:
	BOOL RegenerateData();
	BOOL SaveData();
	virtual ~CData();

protected:
	int Regenerate_Flag;
	void CopyString(char *des, CString src);
};

//{{AFX_INSERT_LOCATION}}

#endif // !defined(AFX_DATADISPLAY_H_INCLUDED_)
