/***************************************************************************
 *                                                                         *
 *                               RasTop                                    *
 *                                                                         *
 *  Version:   1.3 based on RasMol 2.7.1                                   *
 *  ========   June 1st, 2000                                              *
 *                                                                         *
 *  Author:    Philippe Valadon                                            *
 *  =======                                                                *
 *                                                                         *
 *  RasTop is an interactive software to visualize molecular structures    *
 *  based on RasMol Molecular Visualization Tool scripting code. RasTop    *
 *  wraps a friendly graphic interface around RasMol core engine and is    *
 *  intended to promote the use of RasMol as a langage to communicate,     *
 *  transfert, exchange information or teaching purpose among the          *
 *  scientific community.                                                  *
 *                                                                         *
 *  RasTop source is distributed freely. Before using RasTop, contributing *
 *  to the source, or using any of its material, you must read and comply  *
 *  with the NOTICE located in the Help folder of RasTop 1.3 package.      *
 *  The NOTICE applies to RasTop 1.3 as a whole and to the works included  *
 *  within it.                                                             *
 *                                                                         *
 *  Copyrights:                                                            *
 *  ==========                                                             *
 *  RasTop                                                                 *
 *  ------                                                                 *
 *  RasTop version 1.3, August 2000                                        *
 *  Philippe Valadon, P.O. Box 13014, La Jolla, CA 92039, USA              *
 *  valadon@ix.netcom.com                                                  *
 *  Copyright (C) Philippe Valadon 2000                                    *
 *                                                                         *
 *  RasMol                                                                 *
 *  ------                                                                 *
 *  RasMol 2.6 by Roger Sayle                                              *
 *  Biomolecular Structures Group, Glaxo Wellcome Research & Development,  *
 *  Stevenage, Hertfordshire, UK                                           *
 *  Version 2.6, August 1995, Version 2.6.4, December 1998                 *
 *  Copyright (C) Roger Sayle 1992-1999                                    *
 *                                                                         *
 *  Mods by Arne Mueller                                                   *
 *  Version 2.6x1, May 1998                                                *
 *  Copyright (C) Arne Mueller 1998                                        *
 *                                                                         *
 *  Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein                      *
 *  Bernstein + Sons, P.O. Box 177, Bellport, NY, USA                      *
 *  yaya@bernstein-plus-sons.com                                           *
 *  2.7.0 March 1999, 2.7.1 June 1999                                      *
 *  Copyright (C) Herbert J. Bernstein 1998-1999                           *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  If you would like to use major pieces of RasTop in other programs,     *
 *  make modifications to RasTop, or in some other way make what a lawyer  *
 *  would call a "derived work", you are not only permitted to do so, you  *
 *  are encouraged to do so to the condition of respecting the following   *
 *  points:                                                                *
 *   - explain in your documentation how what you did differs              *
 *     from this version of RasTop;                                        *
 *   - make your modified source code available;                           *
 *   - comply with respect to RasTop NOTICE.                               *
 *                                                                         *
 *  Read the full NOTICE for complete information.                         *
 *                                                                         *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// mainfrm.h 
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H_INCLUDED_)
#define AFX_MAINFRM_H_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define		SELECTION_EMPTY		0
#define		SELECTION_NEW		1
#define		SELECTION_ADD		2
#define		SELECTION_REMOVE	3
#define		SELECTION_EXTRACT	4
#define		SELECTION_EXCLUDE	5

#include "pannel.h"
#include "selectbar.h"
#include "colorbar.h"


/////////////////////////////////////////////////////////////////////////////
// CMainFrame mainframe

class CMainFrame : public CMDIFrameWnd
{
	DECLARE_DYNAMIC(CMainFrame)
public:
	CMainFrame();

// Attributes
public:

// Operations
public:
	// Overrides
	//{{AFX_VIRTUAL(CMainFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMainFrame();
	void DockSelectBar();
	void SetSelection( int nMode, CString& sExpression);

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected: 
	CToolBar		  m_wndToolBar;
	CStatusBar		  m_wndStatusBar;

public:	
	int			m_WndRestoreInit;
	int			m_CommandSize;
	int			m_CommandId;
	int			m_CopyZoom;
	CString		m_HelpLine;
	int			m_MenuPicker;
	CPannelBar  m_wndCommandBar;
	CSelectBar	m_wndSelectBar;
	CColorBar	m_wndColorBar;
	CString		m_SelectString;
	COLORREF	m_crColour;
	CString		m_szCommandText;
	int			InfoSilentFlag;

	void UpdateAllViews( CRasView* pIni);
	void ActivateCommandBar();
	void DesactivateCommandBar();
	CRasView* GetActiveRasView();
	CRect GetScreenClientCoor();
	void EnableMenus( int flag );

protected:

	int CreateCommandBar( UINT nID );

	//{{AFX_MSG(CMainFrame)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSysCommand( UINT nID, LPARAM lParam );	
	afx_msg void OnUpdateViewCommandBar(CCmdUI* pCmdUI);
	afx_msg void OnViewCommandBar();
	afx_msg LRESULT OnChangeColor(WPARAM wParam, LPARAM lParam);
	afx_msg void OnSelectionNew();
	afx_msg void OnSelectionAdd();
	afx_msg void OnSelectionRemove();
	afx_msg void OnSelectionExtract();
	afx_msg void OnSelectionExclude();
	afx_msg void OnUpdateSelectionNew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionAdd(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionRemove(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionExtract(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionExclude(CCmdUI* pCmdUI);
	afx_msg void OnViewSelectbar();
	afx_msg void OnUpdateViewSelectbar(CCmdUI* pCmdUI);
	afx_msg void OnViewColorbar();
	afx_msg void OnUpdateViewColorbar(CCmdUI* pCmdUI);
	afx_msg void OnClose();
	afx_msg void OnViewStatusBar();
	afx_msg void OnUpdateViewStatusBar(CCmdUI* pCmdUI);
	afx_msg void OnFilePreferences();
	afx_msg void OnDropFiles(HDROP hDropInfo);
	afx_msg void OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI);
	afx_msg void OnViewMenus();
	afx_msg void OnViewToolbar();
	afx_msg void OnUpdateViewToolbar(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewMenu(CCmdUI* pCmdUI);
	//}}AFX_MSG

protected:
	afx_msg LRESULT UpdateAllViews(WPARAM wParam, LPARAM lParam);
	afx_msg void OnCommandColor(UINT nID);
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}

#endif // !defined(AFX_MAINFRM_H_INCLUDED_)
