/***************************************************************************
 *                                                                         *
 *  RasTop version 1.3, August 2000                                        *
 *  Copyright (C) Philippe Valadon 2000                                    *
 *                                                                         *
 *  Please, read full notice in corresponding header file.                 *
 *                                                                         *
 ***************************************************************************/

////////////////////////////////////////////////////////////////////////
// rasdoc.cpp 
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RasTop.h"
#include <limits.h>
#include <direct.h>
#include <stdlib.h>

#include "dialog.h"
class CRasView;	//Forward declaration
#include "mainfrm.h"
#include "rasdoc.h"
#include "rasview.h"
#include "dialog.h"
#include "graphics.h"

static char snamebuf[1024];
static char fnamebuf[1024];

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static int filetostring( CString & str, LPCSTR filepathname )
{	
	CFile Datafile;
	CFileException fe;
	if ( !Datafile.Open
			( filepathname, CFile::modeRead | CFile::shareDenyWrite, &fe) )
	{	WriteString
			("Warning: File failure in opening molecular data!\n");
		return false;
	}

	BeginWait();

	TRY
	{	int len = (int) Datafile.GetLength();
		char *ptr = str.GetBuffer(len + 1);
		Datafile.Read((void*) ptr, len);
		str.ReleaseBuffer(len);
		Datafile.Close();
	}
	CATCH (CFileException, eLoads)
	{	Datafile.Abort();
		EndWait();
		WriteString
			("Warning: File failure in transfering molecular information!\n");
		return false;
	}
	AND_CATCH (CMemoryException, eData)
	{	Datafile.Abort();
		EndWait();
		WriteString("Warning: Memory error in file operation!\n");
		return false;
	}
	END_CATCH

	//Fix absence of end of line in some files
	int n = 0;
	int r = 0;
	char *ptr;
	ptr = str.LockBuffer();
	while( *ptr )
		if( *ptr++ == '\n' )
			n++;
	str.UnlockBuffer();

	ptr = str.LockBuffer();
	while( *ptr )
		if( *ptr++ == '\r' )
			r++;
	str.UnlockBuffer();

	if( n==0 && r!=0 )
	{	ptr = str.LockBuffer();
		while( *ptr )
			if( *ptr++ == '\r' )
				*(ptr - 1) = '\n';
	}

	//remove script
	int p,q;
	r = 0;
	//rsm file
	p = str.Find("#!rasmol -rsm file");
	if( p!=-1 )
		str = str.Left(p);
	//script
	p = str.Find("#!rasmol -script");
	q = str.Find("exit");
	if( q==-1 )
		q = str.Find("quit");
	if( q!=-1 && p<q )
	{	str = str.Left(p) + str.Right(str.GetLength()-q-5);
	} else if( q!=-1 && q<p )
	{	q = str.Find("HEADER");
		if( p<q )
			str = str.Left(p) + str.Right(str.GetLength()-q);
		else
			r = 1;
	} else if( q!=-1 )
		r = 1;

	if( r==1 )
	{	MessageBox(NULL,"You must edit the molecular file\nto remove the script manually!",
		"Warning", MB_OK|MB_SYSTEMMODAL);
	}

	//remove extra returns
	while(1)
	{	p = str.GetLength();
		if( p==0 ) break;
		if( str.GetAt(0)=='\n'||str.GetAt(0)=='\r' )
			str = str.Right(p-1);
		else break;
	}
	while(1)
	{	p = str.GetLength();
		if( p==0 ) break;
		if( str.GetAt(p-1)=='\n'||str.GetAt(p-1)=='\r' )
			str = str.Left(p-1);
		else break;
	}

	EndWait();
	return true;
}

/////////////////////////////////////////////////////////////////////////////
// CRasDoc

IMPLEMENT_DYNCREATE(CRasDoc, CDocument)

BEGIN_MESSAGE_MAP(CRasDoc, CDocument)
	//{{AFX_MSG_MAP(CRasDoc)
	ON_COMMAND(ID_HELP_RASMOL, OnHelpRasmol)
	ON_COMMAND(ID_SAVE_MOLECULARFILE, OnSaveMolecularfile)
	ON_UPDATE_COMMAND_UI(ID_SAVE_MOLECULARFILE, OnUpdateSaveMolecularfile)
	ON_COMMAND(ID_SAVE_IMAGE, OnSaveImage)
	ON_UPDATE_COMMAND_UI(ID_SAVE_IMAGE, OnUpdateSaveImage)
	ON_COMMAND(ID_SAVE_SCRIPT, OnSaveScript)
	ON_UPDATE_COMMAND_UI(ID_SAVE_SCRIPT, OnUpdateSaveScript)
	ON_COMMAND(ID_FILE_LOAD_RASMOLSCRIPT, OnFileLoadRasmolscript)
	ON_COMMAND(ID_FILE_LOAD_MOLECULARFILE, OnLoadMolecularfile)
	ON_COMMAND(ID_FILE_OPEN_RASTOP, OnFileOpenFromDoc)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRasDoc construction/destruction

CRasDoc::CRasDoc()
{
	ViewActivationFlag = 2;
	m_PixMap = NULL;
	m_BitInfo = NULL;
#ifdef EIGHTBIT			
	m_Palette = NULL;
	m_ColourMap = NULL;
#endif
	m_FBufHandle = NULL;
	m_FileData = "";
}

//static variables
int CRasDoc::m_MoleculeCount = 0;

CRasDoc::~CRasDoc()
{	if( m_FBufHandle != 0 ) 
		GlobalFree(m_FBufHandle);
#ifdef EIGHTBIT			
	if( m_Palette )
		free(m_Palette);
#endif
	if( m_BitInfo )
		free(m_BitInfo);
}

void CRasDoc::SetDocStatus( BOOL Status )
{	GETFRAME

	if( Status )
	{	if( m_FileData.IsEmpty() )
		{	WriteString("This file does not contain any molecular data!\n");
		} else if( Database )
		{	m_MoleculeCount++;
		
#ifdef _DEBUG
			WriteString(">>>> molecule count ++ <<<<\n");
#endif //_DEBUG
					
			pFrame->InfoSilentFlag = true;
			//Select all
			sprintf(CurLine,"select all\n");
			::ExecuteCommand();
			//Reset RASMOLSETPREV
			sprintf(CurLine,"define RASMOLSETPREV selected\n");
			::ExecuteCommand();	
			pFrame->InfoSilentFlag = false;

			pFrame->m_wndSelectBar.ResetBox(true);
		} else
			WriteString("Warning: Molecular data not valid!\n");

		SetModifiedFlag(true);
	} else
	{	ManageDisplay(true);
		if( Database )
		{	m_MoleculeCount--;
		
#ifdef _DEBUG
			WriteString(">>>> molecule count -- <<<<\n");
#endif //_DEBUG
					
			FixRange();
			ZapDatabase();
#ifdef EIGHTBIT			
 			if( ColourMap )
 			{	::DeleteObject(ColourMap);
 				ColourMap = NULL;
 			}
#endif

			/*Free all memory allocations*/
			if( m_MoleculeCount==0 )
			{	ResetVoxelData();
				PurgeDatabase();
				UpdateRender(2);
				UpdateAbstree();
				UpdateRepres();
#ifdef _DEBUG
				WriteString(">>>> DATABASE PURGED <<<<\n");
#endif //_DEBUG
			}
		}
		pFrame->m_wndSelectBar.ResetBox(false);
	}
}

/////////////////////////////////////////////////////////////////////////////
// Switching documents

//Manage the switch of RasMol between documents
void CRasDoc::ManageDisplay(BOOL ActivationFlag)
{	
	GETFRAME
	
	/*Activation of the view*/
	if( ActivationFlag )	
	{	//Only one Activation
		if( ViewActivationFlag != 1 )
			ViewActivationFlag = 1;
		else
			return;

		CheckActivationState();
		TRACE1( "TRACE\tView Activation\t%s\n", GetTitle() );

	    if( m_FBufHandle ) 
			GlobalFree(m_FBufHandle);
#ifdef EIGHTBIT			
		if( m_Palette )
		{	free(m_Palette);
			m_Palette = NULL;
		}
#endif
		if( m_BitInfo )
		{	free(m_BitInfo);
			m_BitInfo = NULL;
		}
#ifdef EIGHTBIT			
		m_ColourMap = NULL;
#endif
		m_PixMap = NULL;

		//regenerate data
		m_DataDisplay.RegenerateData();

		if ( Database )
			ReDrawFlag |= RFRefresh | RFColour | RFInitial;

		//Add defined atom sets
		pFrame->m_wndSelectBar.ResetBox(true);
		//update view and bars
		GETVIEW
		if( pView!=NULL )	//May be NULL on wrong opening by drag&drop
			pView->RefreshView();

	/*Deactivation of the view*/
	} else		
	{	//Only one deactivation
		if (ViewActivationFlag != 0)
			ViewActivationFlag = 0;
		else
			return;

		TRACE1( "TRACE\tView Deactivation\t%s\n", GetTitle() );

		FixRange();
		HDC hDC = ::GetDC(NULL);
#ifdef EIGHTBIT			
 		HPALETTE OldCMap;
 		if( ColourMap )
 		{   int size = sizeof(LOGPALETTE) + 256*sizeof(PALETTEENTRY);
 			m_Palette = (LOGPALETTE *)malloc( size );
 			size = sizeof(BITMAPINFOHEADER) + 256*sizeof(RGBQUAD);
 			m_BitInfo = (BITMAPINFO *)malloc( size );
 			if(m_BitInfo && m_Palette)
 			{	*m_BitInfo = *BitInfo;
 				for(int i=0;i<256;i++)
 					m_BitInfo->bmiColors[i] = BitInfo->bmiColors[i];
 				*m_Palette = *Palette;
 				m_ColourMap = ::CreatePalette(m_Palette);
 				OldCMap = ::SelectPalette(hDC,m_ColourMap,FALSE);
 				::RealizePalette(hDC);
 			} else
 			{	WriteString("Warning: Low memory, you must close documents!\n");
 				return;
 			}
 		}
#else		
		int size = sizeof(BITMAPINFOHEADER);
		m_BitInfo = (BITMAPINFO *)malloc( size );
 		if( m_BitInfo )
			*m_BitInfo = *BitInfo;
#endif

		if( PixMap && m_BitInfo )
		{	long size = (long)XRange*YRange*sizeof(Pixel)+16;
			m_FBufHandle = ::GlobalAlloc(GMEM_MOVEABLE,size);
			if(m_FBufHandle)
			{	LPSTR CopyBBuffer = (LPSTR)::GlobalLock((HGLOBAL) m_FBufHandle); 
				LPSTR CopyABuffer = (LPSTR)::GlobalLock((HGLOBAL) FBufHandle); 
				memcpy(CopyBBuffer,CopyABuffer, size);

				m_PixMap = CreateDIBitmap( hDC, 
						  (BITMAPINFOHEADER __far *)m_BitInfo, 
			               CBM_INIT, CopyABuffer, m_BitInfo, DIB_RGB_COLORS);       
				GlobalUnlock(CopyABuffer);
				GlobalUnlock(CopyBBuffer);
			} else
			{	CRasView* pExView = pFrame->GetActiveRasView();
				pExView->InvalidateRect(NULL, true);
				WriteString("Warning: Low memory, you must close documents!\n");
			}
		}

#ifdef EIGHTBIT			
		if( m_ColourMap && OldCMap )                         
		    ::SelectPalette(hDC,OldCMap,False);
#endif
		::ReleaseDC(NULL,hDC);

		//Save RasMol data, reset SelectBox, stop automode if on
		m_DataDisplay.SaveData();
		pFrame->m_wndSelectBar.ResetBox(false);
		pFrame->m_wndCommandBar.ResetAutoMode();

		ZapDatabase();
	}
}

void CRasDoc::CheckActivationState()
{	/*Ensure that all Rasmol data are deactivated before activating
	  the current one.  Usefull when creating a new document where 
	  the new view is activated before the last one is deactivated*/

	CRasDoc* pDoc;
	CDocTemplate* pDocTemp;
	pDocTemp = this->GetDocTemplate( );
	POSITION posdoc = pDocTemp->GetFirstDocPosition();
	while( posdoc != 0 )
	{	pDoc = (CRasDoc *)pDocTemp->GetNextDoc(posdoc);
		if( pDoc->ViewActivationFlag && (pDoc != this) )
		{	pDoc->ManageDisplay(false);
			TRACE1("TRACE\tActivation defect in CRasDoc\t%s\n", GetTitle());
		}
	}
}

void CRasDoc::FixRange()
{	/*Initialize Range according to saved XRange and YRange.
	  Done systematically before zapping the database.
	*/

	XRange = m_DataDisplay.Data_XRange;
	YRange = m_DataDisplay.Data_YRange;
	Range =  MinFun(XRange,YRange);
	HRange = YRange>>1;
	WRange = XRange>>1;
}

/////////////////////////////////////////////////////////////////////////////
// CRasDoc diagnostics

#ifdef _DEBUG
void CRasDoc::AssertValid() const
{	CDocument::AssertValid();	}

void CRasDoc::Dump(CDumpContext& dc) const
{	CDocument::Dump(dc);		}
#endif //_DEBUG

////////////////////////////////////////////////////////////////////
/*File*/

void CRasDoc::OnFileOpenFromDoc() 
{	static CRasDoc* pDoc = NULL;
	CDocTemplate* pDocTemp;
	POSITION posdoc;
	int countdoc;

	if( pDoc==NULL )
		pDoc = this;

	CheckActivationState();

	GETFRAME

	if( !this->m_FileData.IsEmpty() )
	{	//check first if any empty document
		pDocTemp = this->GetDocTemplate( );
		posdoc = pDocTemp->GetFirstDocPosition( );
		countdoc = 0;
		while( posdoc!=NULL )
		{	pDoc = (CRasDoc *)pDocTemp->GetNextDoc(posdoc);
			if( !pDoc->m_FileData.IsEmpty() )
				pDoc = NULL;
			else
				break;
		}

		if( pDoc==NULL )
		{	//create new document
			((CRasTopApp*) AfxGetApp( ))->OnFileNewRasDoc();
			GETVIEW
			if( pView==NULL )
			{	WriteString("Warning: Failure in opening!\n");
				pDoc = NULL;
				return;
			}
			pDoc = pView->GetDocument();
			if( pDoc==NULL ) 
			{	WriteString("Warning: Failure in opening!\n");
				return;
			}
		} else
		{	//activate the view
			POSITION posview = pDoc->GetFirstViewPosition( );
			CRasView *pView = (CRasView *)pDoc->GetNextView( posview );
			ASSERT( posview == 0 && pView != NULL );
			CMDIChildWnd *pChild = (CMDIChildWnd *)pView->GetParentFrame( );
			pChild->MDIActivate();
		}

		pDoc->OnFileOpenFromDoc();
		return;
   }
		
	char ofilters[128];
    char* dst;
    *fnamebuf = '\0';

   /* filter for file opening */
    dst = ofilters;      
    dst = RegisterFormat(dst,"RSM Scripts (*.rsm)","*.rsm");
    dst = RegisterFormat(dst,"All Files (*.*)","*.*");
    *dst = '\0';

	CFileDialog fdlg(true, NULL, NULL, OFN_HIDEREADONLY | 
	OFN_OVERWRITEPROMPT,  NULL,  NULL);

    fdlg.m_ofn.lStructSize=sizeof(OPENFILENAME);
    fdlg.m_ofn.hwndOwner = NULL;
    fdlg.m_ofn.lpstrFilter = ofilters;
    fdlg.m_ofn.lpstrCustomFilter = NULL;
    fdlg.m_ofn.nFilterIndex = 1;
    fdlg.m_ofn.nMaxFile = 128;
    fdlg.m_ofn.lpstrFileTitle = NULL;
    fdlg.m_ofn.lpstrInitialDir = NULL;
    fdlg.m_ofn.lpstrTitle = "Open";
    fdlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;
    fdlg.m_ofn.lpstrFile = fnamebuf;

    fdlg.m_ofn.lpstrDefExt = "rsm";

	if ( fdlg.DoModal()!=IDOK )
	{	pDoc->OnCloseDocument();
		pDoc = NULL;
		return;
	}

	CString filepath = fdlg.GetPathName();
	if( !pDoc->ManageOpening((LPCTSTR) filepath) )
		WriteString("Warning: Failure in loading data!\n");

	/*Duplicats*/
	CString title = pDoc->GetTitle();
	pDocTemp = this->GetDocTemplate( );
	posdoc = pDocTemp->GetFirstDocPosition( );
	countdoc = 0;
	while( posdoc!=NULL )
	{	CRasDoc* pOtherDoc = (CRasDoc *)pDocTemp->GetNextDoc(posdoc);
		ASSERT(pOtherDoc->IsKindOf(RUNTIME_CLASS(CRasDoc)));
		CString Othertitle = pOtherDoc->GetTitle();
		if( title == Othertitle )
			countdoc++;
	}
	ASSERT(countdoc>=1);
	if( countdoc>1 )	/*itself*/
	{	title += " ~Duplicat";
		pDoc->SetTitle(title);
	}

	//need to refresh duplicats
	GETVIEW
	if( pView!=NULL )
		pView->RefreshView();
	pDoc = NULL;
}

int CRasDoc::ManageOpening(LPCTSTR lpszPathName)
{	GETFRAME
	GETVIEW
		
	BeginWait();
	CString Command = "script ";
	Command += lpszPathName;
	Command += "\n";
	pView->ExecuteCommand(DisplayOff,Command);
	EndWait();

	SetPathName(lpszPathName,true);	
	CString szTemp = lpszPathName;
	int i = szTemp.GetLength()-1;
	while( szTemp.GetAt(i)!='\\' )
	{	if( i==0 )
			break;
		i--;
	}
	szTemp = szTemp.Right(szTemp.GetLength()-i-1);
	SetTitle((LPCSTR) szTemp);

	//Initial reset done before sets were defined
	pFrame->m_wndSelectBar.ResetBox(true);

	//info always NULL since RSM files are scripts
	if( Database!=NULL )
	{	DescribeMolecule();
		//Force background drawing in 32 bits
		FBClear = false;
	}

	SetModifiedFlag(false);
	return true;
}

int CRasDoc::ProcessFileData()
{	BeginWait();
	
	SetDocStatus(false);

	/*Processing molecular data*/
	if( !m_FileData.IsEmpty() )
	{	CFile datafile;
		CFileException fe;
	
		WriteString("Starting data transfer to RasMol!\n");
		if( !datafile.Open("~$tempdata",
			CFile::modeCreate|CFile::modeWrite|CFile::shareDenyWrite, &fe) )
		{	WriteString("Warning: Failure in transferring data to RasMol!\n");
			EndWait();
			return false;
		}

		char *ptr = m_FileData.LockBuffer();
		try
		{	datafile.Write(ptr, m_FileData.GetLength());
			datafile.Close();
		}
		catch(CException* e)
		{	datafile.Abort();
			WriteString("Warning: Failure in transferring data to RasMol!\n");
		    e->Delete();
			SetDocStatus(false);
			EndWait();
			return false;
		}
		m_FileData.UnlockBuffer();
		
		WriteString("Molecular data transfer to RasMol successful!\n");
		if( !FetchFile(DataFileFormat,true,"~$tempdata") )
		{	WriteString("Warning: Failure in reading molecular data!\n");
			SetDocStatus(false);
			EndWait();
			return false;
		} else

		try	{	CFile::Remove( "~$tempdata" );	}
		catch (CException* e)	{	 e->Delete();	}

		strcpy(Info.filename, (LPCSTR) GetPathName( ) );
		DefaultRepresentation();
	}

	SetDocStatus(true);
	EndWait();
	return true;
}

BOOL CRasDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{	

return( CDocument::OnOpenDocument(lpszPathName) );
//todo	return(ManageOpening(lpszPathName));
}

BOOL CRasDoc::OnNewDocument()
{	if( !CDocument::OnNewDocument() )
		return false;
	GETFRAME
	pFrame->ActivateCommandBar();
	pFrame->m_wndCommandBar.UpdatePannel();
	return true;
}

void CRasDoc::OnCloseDocument() 
{	
	SetDocStatus(false);

	/*No pannel if no view*/
	CRasDoc* pDoc;
	CDocTemplate* pDocTemp;
	pDocTemp = GetDocTemplate( );
	POSITION posdoc = pDocTemp->GetFirstDocPosition();
	int p = 0;
	while( posdoc!=NULL )
	{	pDoc = (CRasDoc *)pDocTemp->GetNextDoc( posdoc );
		if( pDoc->IsKindOf(RUNTIME_CLASS(CRasDoc)) )
			p++;
	}
	if( p<=1 )
	{	GETFRAME
		pFrame->DesactivateCommandBar();
	}

	/*Disable PickMode if any*/
	SetPickMode(PickIdent);

	CDocument::OnCloseDocument();
}

BOOL CRasDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{	GETFRAME
	GETVIEW
	if( pView==NULL )
	{	WriteString("Warning: Severe failure in saving file!\n");
		return false;
	}

	BeginWait();
	CString Command = "save ";
	Command += lpszPathName;
	Command += "\n";
	pView->ExecuteCommand(DisplayOff,Command);
	EndWait();
	SetModifiedFlag(false);
	return true;
}

BOOL CRasDoc::SaveModified() 
{	static int DupCount = 0;
	CString title = GetTitle();
	int t = title.Find(" ~Duplicat");
	if(t != -1)
	{	DupCount++;
		title = title.Left(t);
		int p = title.Find('.');
		if( p != -1)
			title = title.Left(p);
		title += "~dup";
		char t[3] = {0};
		_itoa(DupCount, t, 10 );
		title = title + t;
		title += ".rsm";
		SetPathName(( LPCTSTR)title, false);
	}
	return CDocument::SaveModified();
}

void CRasDoc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{	pCmdUI->Enable(true);		}

void CRasDoc::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{	pCmdUI->Enable(true);		}

void CRasDoc::OnLoadMolecularfile() 
{	GETFRAME
	GETVIEW
	if( !pView )
		return;

	if( m_FileData.IsEmpty() )
	{	WriteString(" \n");
		WriteString("Loading molecular file...\n");
		if ( !LoadInputFile(FormatPDB) )
		{	WriteString("Failure in loading molecular file\n");
			return;
		}
		pView->RefreshView();
   } else
   {	//Create new document
		((CRasTopApp*) AfxGetApp( ))->OnFileNewRasDoc();
		pView = pFrame->GetActiveRasView();
		if( pView==NULL )
			return;
		CRasDoc* pDoc = pView->GetDocument();
		if( pDoc!=NULL )
			pDoc->OnLoadMolecularfile();
   }
}

void CRasDoc::OnFileLoadRasmolscript() 
{	char ifilters[128];
    char* dst;
	*fnamebuf = '\0';

	GETFRAME
	GETVIEW
	if( pView==NULL )
		return;

	WriteString("Loading script file...\n");

	dst = ifilters; 
	dst = RegisterFormat(dst,"RasMol Script Files (*.scr)","*.scr");
	dst = RegisterFormat(dst,"Top Script Files (*.top)","*.top");
	dst = RegisterFormat(dst,"All Files (*.*)","*.*");
	*dst = '\0';

	CFileDialog fdlg(true, NULL, NULL, 
					  OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, NULL,  NULL);

	fdlg.m_ofn.lStructSize=sizeof(OPENFILENAME);
	fdlg.m_ofn.hwndOwner = NULL;
	fdlg.m_ofn.lpstrFilter = ifilters;
	fdlg.m_ofn.lpstrCustomFilter = NULL;
	fdlg.m_ofn.nFilterIndex = 1;
	fdlg.m_ofn.nMaxFile = 128;
	fdlg.m_ofn.lpstrFileTitle = NULL;
	fdlg.m_ofn.lpstrInitialDir = NULL;
	fdlg.m_ofn.lpstrTitle = "Select RasMol Script File";
	fdlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;
	fdlg.m_ofn.lpstrFile = fnamebuf;
	fdlg.m_ofn.lpstrDefExt = "scr";
	
	if( fdlg.DoModal()!=IDOK )
	{	WriteString("Loading script interrupted!\n");
		return;
	}

	CString Command = fdlg.GetPathName();
	Command ="script " + Command + "\n";
	pView->ExecuteCommand(DisplayOn,Command);
	
	if( !IsPaused )
		WriteString("Script executed!\n");
}

/*Export*/
void CRasDoc::OnSaveImage() 
{	CDialogImage dlg;
	dlg.m_SizeX = m_DataDisplay.Data_XRange;
	dlg.m_SizeY = m_DataDisplay.Data_YRange;
	if( dlg.DoModal() != IDOK )
		return;

	CString ext;
	int num;
	switch( dlg.m_Format )
	{	case(0)	:	ext = "bmp";  num=1;	break;
		case(1)	:	ext = "gif";  num=2;	break;
		case(2)	:	ext = "ps";	  num=3;	break;
		case(3)	:	ext = "ps";	  num=4;	break;
		case(4)	:	ext = "ps";	  num=5;	break;
		case(5)	:	ext = "ppm";  num=6;	break;
		case(6)	:	ext = "ppm";  num=7;	break;
		case(7)	:	ext = "pic";  num=8;	break;
		case(8)	:	ext = "rgb";  num=9;	break;
		case(9)	:	ext = "ras";  num=10;	break;
		case(10):	ext = "ras";  num=11;	break;
	}

	char filename[128];
	CString fn = GetTitle();
	if( fn.Find('.')!= -1 )		fn = fn.Left(fn.Find('.'));
	if( fn.IsEmpty() )			fn = "notitle";
	if( fn.GetLength()>24 )		fn = fn.Left(24);
	if( !ext.IsEmpty() )		fn += "." + ext;
	fn.MakeLower( );
	strcpy(filename,fn.LockBuffer());
	fn.UnlockBuffer();

	char ofilters[512];
	char* dst;
	dst = ofilters;
    dst = RegisterFormat(dst,"Microsoft Bitmap","*.bmp");
    dst = RegisterFormat(dst,"CompuServe GIF","*.gif");
    dst = RegisterFormat(dst,"Colour PostScript","*.ps;*.eps");
    dst = RegisterFormat(dst,"Mono PostScript","*.ps;*.esp");
    dst = RegisterFormat(dst,"Vector PostScript","*.ps;*.eps");
    dst = RegisterFormat(dst,"Raw Portable Pixmap","*.ppm");
    dst = RegisterFormat(dst,"ASCII Portable Pixmap","*.ppm");
    dst = RegisterFormat(dst,"Apple Macintosh PICT","*.pic");
    dst = RegisterFormat(dst,"Silicon Graphics RGB","*.rgb");
    dst = RegisterFormat(dst,"RLE Sun Rasterfile","*.ras;*.im8");
    dst = RegisterFormat(dst,"Sun Rasterfile","*.ras");
    dst = RegisterFormat(dst,"All Files (*.*)","*.*");
    *dst = '\0';

	CFileDialog fdlg(false, NULL, (LPCSTR) fn, OFN_HIDEREADONLY | 
		OFN_NOREADONLYRETURN | OFN_OVERWRITEPROMPT ,  NULL,  NULL);

    fdlg.m_ofn.lStructSize=sizeof(OPENFILENAME);
    fdlg.m_ofn.hwndOwner = NULL;
    fdlg.m_ofn.lpstrFilter = ofilters;
    fdlg.m_ofn.lpstrCustomFilter = NULL;
    fdlg.m_ofn.nFilterIndex = num;
    fdlg.m_ofn.lpstrDefExt = ext;
    fdlg.m_ofn.nMaxFile = 128;
    fdlg.m_ofn.lpstrFileTitle = filename;
    fdlg.m_ofn.lpstrInitialDir = NULL;
    fdlg.m_ofn.lpstrTitle = "Select Directory and Filename";
    fdlg.m_ofn.Flags |= OFN_PATHMUSTEXIST;

	if (fdlg.DoModal() != IDOK)
		return;

	BeginWait();

	BOOL changedim = false;
	if( (XRange != dlg.m_SizeX) || (YRange != dlg.m_SizeY) ) 
	{	changedim = true;
		XRange = dlg.m_SizeX;
		int dx;
		if( dx = XRange%4 )
			XRange += 4-dx;
		YRange = dlg.m_SizeY;
		Range =  MinFun(XRange,YRange);
		HRange = YRange>>1;
		WRange = XRange>>1;
		ReDrawFlag |= RFReSize;
		ClearImage();
		RefreshScreen();
	}

	fn = fdlg.GetPathName();
	CString sext = fdlg.GetFileExt();
	int sresult = 0;
	if( dlg.m_Format == (int)(fdlg.m_ofn.nFilterIndex -1) )
	{	switch( dlg.m_Format )
		{   case(0):  sresult = WriteBMPFile((LPTSTR)fn.LockBuffer( ));             break;
		    case(1):  sresult = WriteGIFFile((LPTSTR)fn.LockBuffer( ));             break;
		    case(2):  sresult = WriteEPSFFile((LPTSTR)fn.LockBuffer( ),True,True);  break;
		    case(3):  sresult = WriteEPSFFile((LPTSTR)fn.LockBuffer( ),False,True); break;
		    case(4):  sresult = WriteVectPSFile((LPTSTR)fn.LockBuffer( ));          break;
		    case(5):  sresult = WritePPMFile((LPTSTR)fn.LockBuffer( ),True);        break;
		    case(6):  sresult = WritePPMFile((LPTSTR)fn.LockBuffer( ),False);       break;
		    case(7):  sresult = WritePICTFile((LPTSTR)fn.LockBuffer( ));            break;
		    case(8):  sresult = WriteIRISFile((LPTSTR)fn.LockBuffer( ));            break;
		    case(9):  sresult = WriteRastFile((LPTSTR)fn.LockBuffer( ),True);       break;
		    case(10): sresult = WriteRastFile((LPTSTR)fn.LockBuffer( ),False);      break;
		}
	} else if( sext == "bmp" )
	{	sresult = WriteBMPFile((LPTSTR)fn.LockBuffer( ));  
	} else if( sext == "gif" )
	{	sresult = WriteGIFFile((LPTSTR)fn.LockBuffer( )); 
	} else if( sext == "pic" )
	{	sresult = WritePICTFile((LPTSTR)fn.LockBuffer( ));   
	} else if( !((fdlg.m_ofn.nFilterIndex == 10) && sext.IsEmpty()) )
	{	switch(fdlg.m_ofn.nFilterIndex -1)
		{   case(0):  sresult = WriteBMPFile((LPTSTR)fn.LockBuffer( ));             break;
		    case(1):  sresult = WriteGIFFile((LPTSTR)fn.LockBuffer( ));             break;
		    case(2):  sresult = WriteEPSFFile((LPTSTR)fn.LockBuffer( ),true,true);  break;
		    case(3):  sresult = WriteEPSFFile((LPTSTR)fn.LockBuffer( ),false,true); break;
		    case(4):  sresult = WriteVectPSFile((LPTSTR)fn.LockBuffer( ));          break;
		    case(5):  sresult = WritePPMFile((LPTSTR)fn.LockBuffer( ),true);        break;
		    case(6):  sresult = WritePPMFile((LPTSTR)fn.LockBuffer( ),false);       break;
		    case(7):  sresult = WritePICTFile((LPTSTR)fn.LockBuffer( ));            break;
		    case(8):  sresult = WriteIRISFile((LPTSTR)fn.LockBuffer( ));            break;
		    case(9):  sresult = WriteRastFile((LPTSTR)fn.LockBuffer( ),true);       break;
		    case(10): sresult = WriteRastFile((LPTSTR)fn.LockBuffer( ),false);      break;
		}
	} else
		sresult = 0;
	if( !sresult )
		WriteString("RasTop Warning: Unable to save a file!\n");

	fn.UnlockBuffer( );

	if( changedim ) 
	{	FixRange();
		ReDrawFlag |= RFReSize;
		ClearImage();
		RefreshScreen();
	}
	EndWait();
}

void CRasDoc::OnUpdateSaveImage(CCmdUI* pCmdUI) 
{	pCmdUI->Enable( (int)Database );	}

void CRasDoc::OnSaveMolecularfile() 
{	CDialogSaveFile dlg;
	if( dlg.DoModal() != IDOK )
		return;
	CString ext;
	int num;
	switch( dlg.m_Format )
	{	case(0)	:	ext = "pdb";	num = 1;	break;
		case(1)	:	ext = "mdl";	num = 2;	break;
		case(2)	:	ext = "xyz";	num = 3;	break;
		case(3)	:	ext = "cif";	num = 4;	break;
	}

	CString Filename = GetTitle();
	int d = Filename.Find('.');
	if(d != -1) Filename = Filename.Left(d);
	if(Filename.IsEmpty()) Filename = "notitle";
	if(Filename.GetLength()>24) Filename = Filename.Left(24);
	Filename += "." + ext;

	char ofilters[512];
	char* dst;
	dst = ofilters;
    dst = RegisterFormat(dst,"Brookhaven Databank (*.pdb;*.ent)","*.pdb;*.ent");
    dst = RegisterFormat(dst,"MDL Mol File Format (*.mdl;*.mol)","*.mdl;*.mol");
    dst = RegisterFormat(dst,"MSC (XMol) XYZ Format (*.xyz)","*.xyz");
    dst = RegisterFormat(dst,"Crystallographic Information File (*.cif)","*.cif");
    dst = RegisterFormat(dst,"All Files (*.*)","*.*");
    *dst = '\0';

	CFileDialog fdlg(false, NULL, (LPCSTR) Filename, OFN_HIDEREADONLY | 
		OFN_NOREADONLYRETURN | OFN_OVERWRITEPROMPT ,  NULL,  NULL);

    fdlg.m_ofn.lStructSize=sizeof(OPENFILENAME);
    fdlg.m_ofn.hwndOwner = NULL;
    fdlg.m_ofn.lpstrFilter = ofilters;
    fdlg.m_ofn.lpstrCustomFilter = NULL;
    fdlg.m_ofn.nFilterIndex = num;
    fdlg.m_ofn.lpstrDefExt = ext;
    fdlg.m_ofn.nMaxFile = 128;
    fdlg.m_ofn.lpstrFileTitle = NULL;
    fdlg.m_ofn.lpstrInitialDir = NULL;
    fdlg.m_ofn.lpstrTitle = "Select Directory and Filename";
    fdlg.m_ofn.Flags |= OFN_PATHMUSTEXIST;

	if( fdlg.DoModal() != IDOK )
		return;

	Filename = fdlg.GetPathName();
	CString sext = fdlg.GetFileExt();
	int sresult = 0;
	if( sext == "pdb" )
		sresult = SavePDBMolecule(( LPTSTR) Filename.LockBuffer());
	else if( sext == "mdl" )
		sresult = SaveMDLMolecule(( LPTSTR) Filename.LockBuffer());
	else if( sext == "xyz" )
		sresult = SaveXYZMolecule(( LPTSTR) Filename.LockBuffer()); 
	else if( sext == "cif" )
		sresult = SaveCIFMolecule(( LPTSTR) Filename.LockBuffer());
	else if( (sext != "pdb")&&(sext != "mdl")&&(sext != "xyz")&&(sext != "cif") )
	{	switch(dlg.m_Format)
		{	case(0)	:	sresult = SavePDBMolecule(( LPTSTR) Filename.LockBuffer());	break;
			case(1)	:	sresult = SaveMDLMolecule(( LPTSTR) Filename.LockBuffer()); break;
			case(2)	:	sresult = SaveXYZMolecule(( LPTSTR) Filename.LockBuffer()); break;
			case(3)	:	sresult = SaveCIFMolecule(( LPTSTR) Filename.LockBuffer()); break;
		}
	}
	if( !sresult )
		WriteString("RasTop Warning: Unable to save a file!\n");
	Filename.UnlockBuffer( );

}

void CRasDoc::OnUpdateSaveMolecularfile(CCmdUI* pCmdUI) 
{	pCmdUI->Enable( (int)Database );	}

void CRasDoc::OnSaveScript() 
{	CDialogScript dlg;
	if( dlg.DoModal() != IDOK )
		return;
	CString ext;
	int num;
	switch( dlg.m_Format )
	{	case(0)	:	ext = "scr";	num = 1;	break;
		case(1)	:	ext = "mol";	num = 2;	break;
		case(2)	:	ext = "kin";	num = 3;	break;
		case(3)	:	ext = "pov";	num = 4;	break;
		case(4)	:	ext = "pov";	num = 5;	break;
		case(5)	:	ext = "vrm";	num = 6;	break;
		case(6)	:	ext = "txt";	num = 7;	break;
		case(7)	:	ext = "txt";	num = 8;	break;
		case(8)	:	ext = "txt";	num = 9;	break;
	}
	
	CString Filename = GetTitle();
	int d = Filename.Find('.');
	if( d != -1 )
		Filename = Filename.Left(d);
	if( Filename.IsEmpty() )
		Filename = "notitle";
	if( Filename.GetLength()>24 )
		Filename = Filename.Left(24);
	Filename += "." + ext;

	char ofilters[512];
	char* dst;
	dst = ofilters;
    dst = RegisterFormat(dst,"Rasmol Script file (*.scr)","*.scr");
    dst = RegisterFormat(dst,"MolScript file (*.mol)","*.mol");
    dst = RegisterFormat(dst,"Kinemage file (*.kin)","*.kin");
    dst = RegisterFormat(dst,"POVRay2 file (*.pov)","*.pov");
    dst = RegisterFormat(dst,"POVRay3 file (*.pov)","*.pov");
    dst = RegisterFormat(dst,"VMRL file (*.vmr)","*.vmr");
    dst = RegisterFormat(dst,"Phipsi (*.txt)","*.txt");
    dst = RegisterFormat(dst,"Ramachandran file (*.txt)","*.txt");
    dst = RegisterFormat(dst,"Ramachandran plot (*.txt)","*.txt");
    dst = RegisterFormat(dst,"All Files (*.*)","*.*");
    *dst = '\0';

	CFileDialog fdlg(false, NULL, (LPCSTR) Filename, OFN_HIDEREADONLY | OFN_NOREADONLYRETURN | 
	OFN_OVERWRITEPROMPT ,  NULL,  NULL);

    fdlg.m_ofn.lStructSize=sizeof(OPENFILENAME);
    fdlg.m_ofn.hwndOwner = NULL;
    fdlg.m_ofn.lpstrFilter = ofilters;
    fdlg.m_ofn.lpstrCustomFilter = NULL;
    fdlg.m_ofn.nFilterIndex = num;
    fdlg.m_ofn.lpstrDefExt = ext;
    fdlg.m_ofn.nMaxFile = 128;
    fdlg.m_ofn.lpstrFileTitle = NULL;
    fdlg.m_ofn.lpstrInitialDir = NULL;
    fdlg.m_ofn.lpstrTitle = "Select Directory and Filename";
    fdlg.m_ofn.Flags |= OFN_PATHMUSTEXIST;

	if( fdlg.DoModal() != IDOK )
		return;

	Filename = fdlg.GetPathName();
	CString sext = fdlg.GetFileExt();
	int sresult = 0;
	if( sext == "scr" )
		sresult = WriteScriptFile((LPTSTR) Filename.LockBuffer(),false);
	else if( sext == "mol" )
		sresult = WriteMolScriptFile((LPTSTR) Filename.LockBuffer());
	else if( sext == "kin" )
		sresult = WriteKinemageFile((LPTSTR) Filename.LockBuffer()); 
	else if( sext == "vrm" )
		sresult = WriteVRMLFile((LPTSTR) Filename.LockBuffer()); 
	else
	{	
	switch( dlg.m_Format )
	{	
	case(0)	:	sresult = WriteScriptFile((LPTSTR) Filename.LockBuffer(),false);
				break;
	case(1)	:	sresult = WriteMolScriptFile((LPTSTR) Filename.LockBuffer());
				break;
	case(2)	:	sresult = WriteKinemageFile((LPTSTR) Filename.LockBuffer());
				break;
	case(3)	:	sresult = WritePOVRayFile((LPTSTR) Filename.LockBuffer());
				break;
	case(4)	:	sresult = WritePOVRay3File((LPTSTR) Filename.LockBuffer());
				break;
	case(5)	:	sresult = WriteVRMLFile((LPTSTR) Filename.LockBuffer());
				break;
	case(6)	:	sresult = WritePhiPsiAngles((LPTSTR) Filename.LockBuffer(), false);
				break;
	case(7)	:	sresult = WritePhiPsiAngles((LPTSTR) Filename.LockBuffer(), 1);
				break;
	case(8)	:	sresult = WritePhiPsiAngles((LPTSTR) Filename.LockBuffer(), -1);
				break;
	}
	}
	if( !sresult )
		WriteString("RasTop Warning: Unable to save a file!\n");
	Filename.UnlockBuffer();
}

void CRasDoc::OnUpdateSaveScript(CCmdUI* pCmdUI) 
{	pCmdUI->Enable( (int)Database );	}


/*Help*/
void CRasDoc::OnHelpRasmol() 
{	/*char *src, *dst;
	if( _getcwd(fnamebuf,1024) )
	{	dst = fnamebuf;
		while( *dst ) dst++;
		if( *(dst-1) != '\\' ) 
			*dst++ = '\\';
		src = "RASWIN.HLP";    
		while( *dst++ = *src++ );
		GETFRAME
		GETVIEW
		WinHelp( pView->m_hWnd,fnamebuf,HELP_INDEX,0L );
	}
	*fnamebuf = '\0';
	*/

	GETFRAME
	if( pFrame->m_HelpLine.IsEmpty() )
	{	WriteString("Warning: No indication where to find help!\n");
		WriteString("...      Check preferences!\n");
	} else
	{	::ShellExecute(NULL, NULL, pFrame->m_HelpLine, 
			NULL, _T("C:\\"),SW_SHOWNORMAL);
	}
}


///////////////////////////////////////////////////////////////////////////////////

char *RegisterFormat( char *buffer, char *desc, char *ext )
{
    while( *buffer++ = *desc++ );
    while( *buffer++ = *ext++ );
    return( buffer );
}

BOOL LoadInputFile(int format )
{	static char ifilters[512];
	register char *ext;
    register int num;
    char* dst;
    *fnamebuf = '\0';

    switch( format )
    {   case(FormatPDB):      ext = "PDB";  num = 1;  break;
		case(FormatAlchemy):  ext = "MOL";  num = 2;  break;
		case(FormatMol2):     ext = "MOL";  num = 3;  break;
		case(FormatMDL):      ext = "MOL";  num = 4;  break;
		case(FormatXYZ):      ext = "XYZ";  num = 5;  break;
		case(FormatCharmm):   ext = "CHM";  num = 6;  break;
        case(FormatMOPAC):    ext = "MOP";  num = 7;  break;
		case(FormatCIF):      ext = "CIF";  num = 8;  break;
    }

   /* filter for file opening */
    dst = ifilters;      
    dst = RegisterFormat(dst,"Brookhaven Databank (*.pdb;*.ent)","*.pdb;*.ent");
    dst = RegisterFormat(dst,"Alchemy File Format (*.alc;*.mol)","*.alc;*.mol");
    dst = RegisterFormat(dst,"Sybyl MOL2 Format (*.syb;*.mol)","*.syb;*.mol");
    dst = RegisterFormat(dst,"MDL Mol File Format (*.mdl;*.mol)","*.mdl;*.mol");
    dst = RegisterFormat(dst,"MSC (XMol) XYZ Format (*.xyz)","*.xyz");
    dst = RegisterFormat(dst,"CHARMm File Format (*.chm)","*.chm");
    dst = RegisterFormat(dst,"MOPAC File Format (*.mop)","*.mop");
    dst = RegisterFormat(dst,"CIF File Format  (*.cif)","*.cif");
    dst = RegisterFormat(dst,"All Files (*.*)","*.*");
    *dst = '\0';

	CFileDialog fdlg(true,NULL,NULL,OFN_HIDEREADONLY | 
	OFN_OVERWRITEPROMPT, NULL, NULL);

    fdlg.m_ofn.lStructSize=sizeof(OPENFILENAME);
    fdlg.m_ofn.hwndOwner = NULL;
    fdlg.m_ofn.lpstrFilter = ifilters;
    fdlg.m_ofn.lpstrCustomFilter = NULL;
    fdlg.m_ofn.nFilterIndex = num;
    fdlg.m_ofn.nMaxFile = 128;
    fdlg.m_ofn.lpstrFileTitle = NULL;
    fdlg.m_ofn.lpstrInitialDir = NULL;
    fdlg.m_ofn.lpstrTitle = "Select Molecular Coordinate File";
    fdlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;
    fdlg.m_ofn.lpstrFile = fnamebuf;

    fdlg.m_ofn.lpstrDefExt = ext;

	if (fdlg.DoModal()!=IDOK)
		return false;

	BeginWait( );

	CString cext = fdlg.GetFileExt();
	int Success;
	if( (cext == "pdb")||(cext == "ent") )
	{	Success = FetchFile(FormatPDB,true,fnamebuf);
	} else if( cext == "alc" )
	{	Success = FetchFile(FormatAlchemy,true,fnamebuf);
	} else if( cext == "syb" )
	{	Success = FetchFile(FormatMol2,true,fnamebuf);
	} else if( cext == "mdl" )
	{	Success = FetchFile(FormatMDL,true,fnamebuf); 
	} else if( cext == "xyz" )
	{	Success = FetchFile(FormatXYZ,true,fnamebuf);
	} else if( cext == "chm" )
	{	Success = FetchFile(FormatCharmm,true,fnamebuf); 
	} else if( cext == "mop" )
	{	Success = FetchFile(FormatMOPAC,true,fnamebuf);
	} else if( cext == "cif" )
	{	Success = FetchFile(FormatCIF,true,fnamebuf);
	} else 
	{	switch( fdlg.m_ofn.nFilterIndex )
		{   case(1): Success = FetchFile(FormatPDB,true,fnamebuf);		break;
		    case(2): Success = FetchFile(FormatAlchemy,true,fnamebuf);	break;
		    case(3): Success = FetchFile(FormatMol2,true,fnamebuf);		break;
		    case(4): Success = FetchFile(FormatMDL,true,fnamebuf);		break;
		    case(5): Success = FetchFile(FormatXYZ,true,fnamebuf);		break;
		    case(6): Success = FetchFile(FormatCharmm,true,fnamebuf);	break;
		    case(7): Success = FetchFile(FormatMOPAC,true,fnamebuf);	break;
		    case(8): Success = FetchFile(FormatCIF,true,fnamebuf);		break;
			default: Success = FetchFile(FormatPDB,true,fnamebuf);    
		}
	}

	EndWait( );
	DefaultRepresentation();
	return Success;
}

int SaveDoc( char *filename )
{	BeginWait();

	CMDIFrameWnd *pFrame = (CMDIFrameWnd*)AfxGetApp()->m_pMainWnd;
	CMDIChildWnd *pChild = (CMDIChildWnd *)pFrame->GetActiveFrame();
	CRasDoc *pDoc = (CRasDoc *)pChild->GetActiveDocument( );
	if( pDoc==NULL )
		return false;

	CFile file;
	CFileException fe;

	if ( !file.Open(filename, CFile::modeCreate |
		 CFile::modeReadWrite | CFile::shareExclusive, &fe) )
	{	pDoc->ReportSaveLoadException(filename, &fe, true, AFX_IDP_INVALID_FILENAME);
		return false;
	}

	char *ptr = pDoc->m_FileData.LockBuffer();
	try
	{	file.Write( ptr, pDoc->m_FileData.GetLength() );
		file.Close();
	}
	catch ( CException* eLoad )
	{	file.Abort();
		eLoad->Delete();
		EndWait();
		WriteString("RasTop: Failure in saving molecule data!\n");
		return false;
	}
	EndWait();
	return true;
}

int ProcessDoc( int Status )
{	CMDIFrameWnd *pFrame = (CMDIFrameWnd*)AfxGetApp()->m_pMainWnd;
	CMDIChildWnd *pChild = (CMDIChildWnd *)pFrame->GetActiveFrame();
	CRasDoc *pDoc = (CRasDoc *)pChild->GetActiveDocument( );
	if( pDoc==NULL )
		return true;

	if( Status )
	{	if( !filetostring( pDoc->m_FileData, DataFileName) )
		{	pDoc->m_FileData.Empty();
			WriteString("RasTop Warning: Unable to allocate molecular data!\n");
		}
	} else
		pDoc->m_FileData.Empty();

	pDoc->SetDocStatus(Status);
	return true;
}

