/***************************************************************************
 *                                                                         *
 *                               RasTop                                    *
 *                                                                         *
 *  Version:   1.3 based on RasMol 2.7.1                                   *
 *  ========   June 1st, 2000                                              *
 *                                                                         *
 *  Author:    Philippe Valadon                                            *
 *  =======                                                                *
 *                                                                         *
 *  RasTop is an interactive software to visualize molecular structures    *
 *  based on RasMol Molecular Visualization Tool scripting code. RasTop    *
 *  wraps a friendly graphic interface around RasMol core engine and is    *
 *  intended to promote the use of RasMol as a langage to communicate,     *
 *  transfert, exchange information or teaching purpose among the          *
 *  scientific community.                                                  *
 *                                                                         *
 *  RasTop source is distributed freely. Before using RasTop, contributing *
 *  to the source, or using any of its material, you must read and comply  *
 *  with the NOTICE located in the Help folder of RasTop 1.3 package.      *
 *  The NOTICE applies to RasTop 1.3 as a whole and to the works included  *
 *  within it.                                                             *
 *                                                                         *
 *  Copyrights:                                                            *
 *  ==========                                                             *
 *  RasTop                                                                 *
 *  ------                                                                 *
 *  RasTop version 1.3, August 2000                                        *
 *  Philippe Valadon, P.O. Box 13014, La Jolla, CA 92039, USA              *
 *  valadon@ix.netcom.com                                                  *
 *  Copyright (C) Philippe Valadon 2000                                    *
 *                                                                         *
 *  RasMol                                                                 *
 *  ------                                                                 *
 *  RasMol 2.6 by Roger Sayle                                              *
 *  Biomolecular Structures Group, Glaxo Wellcome Research & Development,  *
 *  Stevenage, Hertfordshire, UK                                           *
 *  Version 2.6, August 1995, Version 2.6.4, December 1998                 *
 *  Copyright (C) Roger Sayle 1992-1999                                    *
 *                                                                         *
 *  Mods by Arne Mueller                                                   *
 *  Version 2.6x1, May 1998                                                *
 *  Copyright (C) Arne Mueller 1998                                        *
 *                                                                         *
 *  Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein                      *
 *  Bernstein + Sons, P.O. Box 177, Bellport, NY, USA                      *
 *  yaya@bernstein-plus-sons.com                                           *
 *  2.7.0 March 1999, 2.7.1 June 1999                                      *
 *  Copyright (C) Herbert J. Bernstein 1998-1999                           *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  If you would like to use major pieces of RasTop in other programs,     *
 *  make modifications to RasTop, or in some other way make what a lawyer  *
 *  would call a "derived work", you are not only permitted to do so, you  *
 *  are encouraged to do so to the condition of respecting the following   *
 *  points:                                                                *
 *   - explain in your documentation how what you did differs              *
 *     from this version of RasTop;                                        *
 *   - make your modified source code available;                           *
 *   - comply with respect to RasTop NOTICE.                               *
 *                                                                         *
 *  Read the full NOTICE for complete information.                         *
 *                                                                         *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// rasdoc.h
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_RASDOC_H_INCLUDED_)
#define AFX_RASDOC_H_INCLUDED_

#include <afxtempl.h>
#include "datadisplay.h"

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

char *RegisterFormat( char *buffer, char *desc, char *ext );
BOOL LoadInputFile(int format);

class CRasDoc : public CDocument
{
protected:
	CRasDoc();
	DECLARE_DYNCREATE(CRasDoc)

// Attributes
public:

// Operations
public:

// Overrides
	//{{AFX_VIRTUAL(CRasDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual void OnCloseDocument();
	protected:
	virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CRasDoc();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

public:
	static int m_MoleculeCount;		//Count number of opened molecules
	CString m_FileData;				//Molecular data

	CData m_DataDisplay;			//Data exchange with rasmol
	int ViewActivationFlag;			//Overcome duplicat of activation/desactivation

	HBITMAP m_PixMap;
	HGLOBAL m_FBufHandle;
#ifdef EIGHTBIT			
	HPALETTE m_ColourMap;
	LOGPALETTE* m_Palette;
#endif
	BITMAPINFO* m_BitInfo;

	void CheckActivationState();
	void FixRange();
	void ManageDisplay(BOOL ActivationFlag);
	int ManageOpening(LPCTSTR lpszPathName);
	int ProcessFileData( );
	void SetDocStatus( BOOL Status );

protected:
	//{{AFX_MSG(CRasDoc)
	afx_msg void OnHelpRasmol();
	afx_msg void OnSaveMolecularfile();
	afx_msg void OnUpdateSaveMolecularfile(CCmdUI* pCmdUI);
	afx_msg void OnSaveImage();
	afx_msg void OnUpdateSaveImage(CCmdUI* pCmdUI);
	afx_msg void OnSaveScript();
	afx_msg void OnUpdateSaveScript(CCmdUI* pCmdUI);
	afx_msg void OnFileLoadRasmolscript();
	afx_msg void OnLoadMolecularfile();
	afx_msg void OnFileOpenFromDoc();
	afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}

#endif // !defined(AFX_RASDOC_H_INCLUDED_)
